
##############################################################################
# Data reduction script for VY CMa Band 7 321-GHz 12-m data: Part 1: Calibration
# VYCMa_Band7_321GHz_Calibration.py
# Tested using CASA Version 4.2.1 (r29048)
#
# Datasets used:
#
# uid___A002_X6cdf83_X1118 - observed Aug 16 2013 with 16 antennas
# Flux scale calibrator J0522-364 (but use value derived from Pallas)
# Bandpass calibrator J0522-3627 (same as J0522-364) 
# Phase reference source J0648-3044 
# Target Vy_CMa
#
# uid___A002_X6cdf83_X12ca - observed Aug 16 2013 with 16 antennas
# Flux scale calibrator Pallas 
# Bandpass calibrator J0522-3627 
# Phase reference source J0648-3044 
# Target Vy_CMa
#
# uid___A002_X6d0f96_X2051 - observed Aug 17 2013 with 20 antennas
# Flux scale calibrator  Pallas
# Bandpass calibrator    J0522-3627
# Phase reference source J0648-3044
# Target                 Vy_CMa
#
##############################################################################
"""
See accompanying README file for details of the necessary input files
and comments on the data.

Prior to running this script you will need to obtain the Analysis Utilities package.
See http://casaguides.nrao.edu/index.php?title=Analysis_Utilities for details and 
for how to download Analysis Utilities.

"""

#----------------------------------------------------------------------------------
#----- Optional Steps -------------------------------------------------------------
#----------------------------------------------------------------------------------

#----- Set this option to true if you want to make diagnostic plots along the way.
#----- Note that this will slow down the reduction somewhat.
#----- Default is no plots (you can make them later anyway).
calplots=T

#----------------------------------------------------------------------------------
#----- Some setup steps -----------------------------------------------------------
#----------------------------------------------------------------------------------

version = casalog.version()
print "You are using " + version
if (version < '4.2.1'):
    print "\033[91m YOUR VERSION OF CASA IS TOO OLD FOR THIS GUIDE."
    print "\033[91m PLEASE UPDATE IT BEFORE PROCEEDING."
else:
    print "Your version of CASA is appropriate for this guide."

# Repeat imports if you exit and restart CASA if using copy and paste
#import re
#import os

thesteps=[]
step_title = {0: 'Import of the ASDMs, listing the data and plotting antenna table',
              1: 'Flag autocorrelations and pointing scans; back up flags',
              2: 'Generation and time averaging of the WVR cal table',
              3: 'Generation of the Tsys cal table',
              4: 'Generation of the antenna position cal table',
              5: 'Plot raw data',
              6: 'Application of the WVR, Tsys and antpos cal tables',
              7: 'Split out science SPWs and time average',
              8: 'Listobs, clear pointing table, and save original flags',
              9: 'Plot data with instrumental calibration',
              10: 'Initial flagging',
              11: 'Bandpass calibration',
              12: 'Putting a model for the flux calibrator',
              13: 'Calibrate the flux scale',
              14: 'Gain calibration',
              15: 'Application of the bandpass and gain cal tables',
              16: 'Plot calibrated phase ref data',
              17: 'Split out the science data'}

# The Python variable 'mysteps' will control which steps
# are executed when you start the script using
#   execfile('scriptForCalibration.py')
# e.g. setting
#   mysteps = [2,3,4]# before starting the script will make the script execute
# only steps 2, 3, and 4
# Setting mysteps = [] will make it execute all steps.

try:
  print 'List of steps to be executed ...', mysteps
  thesteps = mysteps
except:
  print 'global variable mysteps not set.'
if (thesteps==[]):
  thesteps = range(0,len(step_title))
  print 'Executing all steps: ', thesteps

######################################################

# If executing this script by copying and pasting, and you restart
# CASA, copy the definition of basename before executing steps 0-7
basename=['uid___A002_X6cdf83_X1118','uid___A002_X6cdf83_X12ca','uid___A002_X6d0f96_X2051']
# Copy the definition of refant for steps 5-15
refantenna='DV15'

# Import of the ASDMs, listing the data and plotting antenna table
mystep = 0
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  for name in basename:
      os.system('rm -rf '+name+'.ms')
      importasdm(asdm=name, vis=name+'.ms', process_flags=F, 
                 asis='Antenna Station CalWVR Receiver Source')

# Do listobs and write info for each dataset to a text file
  for name in basename:
      os.system('rm '+name+'_listobs.txt')
      listobs(vis=name+'.ms', listfile=name+'.listobs.txt', verbose=True)
      print "<< Printed listobs output to "+name+'.listobs.txt'

# Plot antenna configuration to a file
  for name in basename:
      plotants(name+'.ms', figfile='plotants_'+name+'.png')

  print 'Choose refant - with lots of short baselines but not so close other antennas to\nthat it will be shadowed. Definition is at top of script, change if desired.'

#################################
# Flag autocorrelations and pointing scans; back up flags
mystep = 1 
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

# Remove all previous flags
  for name in basename:
      flagdata(vis=name+'.ms',mode='unflag', flagbackup=F)

  for name in basename:
      flagdata(vis = name+'.ms',
               mode = 'manual',
               spw = '1~24',
               autocorr = T,
               flagbackup = F)
  
  for name in basename:
      flagdata(vis = name+'.ms',
               mode = 'manual',
               intent = '*POINTING*,*SIDEBAND_RATIO*,*ATMOSPHERE*',
               flagbackup = F)

# Store current flagging state
  for name in basename:
      flagmanager(vis = name+'.ms', mode = 'save',
                  versionname = 'Apriori')

# To restore a priori flags:
#for name in basename:
#	flagmanager(vis = name+'.ms', mode = 'restore', versionname = 'Apriori')	

###############
# Generation and time averaging of the WVR cal table
mystep = 2
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]
  
  for name in basename:
      os.system('rm -rf ' +name+'.ms.wvr') 
      mylogfile = casalog.logfile()
      casalog.setlogfile(name+'.ms.wvrgcal')
      wvrgcal(vis = name+'.ms',
              caltable = name+'.ms.wvr',
              toffset = 0,
              tie = ['Vy_CMa,J0648-3044'],
              statsource = 'Vy_CMa')
      casalog.setlogfile(mylogfile)
  
  for name in basename:
      os.system('rm -rf ' +name+'.ms.wvr.smooth') 
      smoothcal(vis = name+'.ms',
                tablein = name+'.ms.wvr',
                caltable = name+'.ms.wvr.smooth',
                smoothtype = 'mean',
                smoothtime = 6.048)
  
  if(calplots):
      print 'Please ignore any messages arising from data sets with <17 antennas'
      for name in basename:
          os.system('rm -rf '+name+'.ms.wvr.smooth*.png')
          for ant in range(3):
              plotcal(caltable = name+'.ms.wvr.smooth',
                      xaxis = 'time', yaxis = 'phase',
                      plotrange = [0,0,-180,180],
                      antenna=str(ant*8)+'~'+str(ant*8+7),
                      iteration = 'antenna',
                      markersize=3.0,
                      figfile=name+'.ms.wvr.smooth_ant'+str(ant*8)+'-'+str(ant*8+7)+'.png',
                      subplot = 421,
                      fontsize=7.0)

# Flag WVR autocorrelations
  for name in basename:
      flagdata(vis = name+'.ms',
               mode = 'manual',
               spw = '0',
               autocorr = T,
               flagbackup = F)
  
#####################################	
# Generation of the Tsys cal table
mystep = 3
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]
 
  for name in basename:
      os.system('rm -rf '+name+'.ms.tsys') 
      gencal(vis = name+'.ms',
             caltable = name+'.ms.tsys',
             caltype = 'tsys')
  
  if(calplots):
      for name in basename:
          os.system('rm -rf '+name+'.ms.tsys*.png')
          for ant in range(3):
              plotcal(caltable = name+'.ms.tsys',
                  xaxis = 'freq', yaxis = 'tsys',
                  spw='9:8~120,11:8~120,13:8~120,15:8~120',
                  markersize=3.0,
                  antenna=str(ant*8)+'~'+str(ant*8+7),
                  iteration = 'antenna',
                  figfile=name+'.ms.tsys_ant'+str(ant*8)+'-'+str(ant*8+7)+'.png',
                  subplot = 421,
                  fontsize=7.0)
 
######################################
# Generation of the antenna position cal table
mystep = 4
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  for name in basename:    
      os.system('rm -rf '+ name+'.ms.antpos') 

  gencal(vis = 'uid___A002_X6cdf83_X1118.ms',
    caltable = 'uid___A002_X6cdf83_X1118.ms.antpos',
    caltype = 'antpos',
      antenna = 'DA46,DA52,DV03,DV04,DV05,DV07,DV11,DV13,DV14,DV15,DV17,DV18,DV19,DV22,DV24,DV25',
      parameter = [0.000006, 0.000228, -0.000297, 0.000068, 0.000018, 0.000008, -0.000196, -0.000061,   0.000178, -0.000132, 0.000341, -0.000190, 0.000068, 0.000267, -0.000494, -0.000524, 0.000838, 0.000626, 0.000112, -0.000116, 0.000112, -0.000032, 0.000188, -0.000261, 0.000043, -0.000173, 0.000008, 0.000037, 0.000468, -0.000484, 0.000167, -0.000474, 0.000167, -0.000047, 0.000177, -0.000151, -0.000282,-0.000304, -0.000094, -0.000185, 0.000227, -0.000283, -0.000404, 0.003222, 0.001466, -0.000296, 0.000305, 0.000940])

  gencal(vis = 'uid___A002_X6cdf83_X12ca.ms',
    caltable = 'uid___A002_X6cdf83_X12ca.ms.antpos',
    caltype = 'antpos',
      antenna = 'DA46,DA52,DV03,DV04,DV05,DV07,DV11,DV13,DV14,DV15,DV17,DV18,DV19,DV22,DV24,DV25',
      parameter = [0.000006, 0.000228, -0.000297, 0.000068, 0.000018, 0.000008, -0.000196, -0.000061,   0.000178, -0.000132, 0.000341, -0.000190, 0.000068, 0.000267, -0.000494, -0.000524, 0.000838, 0.000626, 0.000112, -0.000116, 0.000112, -0.000032, 0.000188, -0.000261, 0.000043, -0.000173, 0.000008, 0.000037, 0.000468, -0.000484, 0.000167, -0.000474, 0.000167, -0.000047, 0.000177, -0.000151, -0.000282,-0.000304, -0.000094, -0.000185, 0.000227, -0.000283, -0.000404, 0.003222, 0.001466, -0.000296, 0.000305, 0.000940])

  gencal(vis = 'uid___A002_X6d0f96_X2051.ms',
    caltable = 'uid___A002_X6d0f96_X2051.ms.antpos',
    caltype = 'antpos',
    antenna = 'DA41,DA46,DA50,DA56,DV03,DV04,DV05,DV07,DV11,DV13,DV14,DV15,DV17,DV18,DV19,DV21,DV22,DV24,DV25',
    parameter = [-0.018216, 0.067084, 0.060448, 0.000006, 0.000228, -0.000297, 0.000068, 0.000018, 0.000008, 0.000065, 0.000035, 0.000115, -0.000196, -0.000061, 0.000178, -0.000132, 0.000341, -0.000190, 0.000068, 0.000267, -0.000494, -0.000524, 0.000838, 0.000626, 0.000112, -0.000116, 0.000112, -0.000032, 0.000188, -0.000261, 0.000043, -0.000173, 0.000008, 0.000037, 0.000468, -0.000484, 0.000167, -0.000474, 0.000167, -0.000047, 0.000177, -0.000151, -0.000282, -0.000304, -0.000094, -0.000185, 0.000023, 0.000032, 0.000009, 0.000227, -0.000283, -0.000404, 0.003222, 0.001466, -0.000296, 0.000305, 0.000940])


#######
# Plot visibilities of bandpass calibrator before applying instrumental calibration
mystep = 5
if(mystep in thesteps):
    casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
    print 'Step ', mystep, step_title[mystep]

    for name in basename:
        os.system('rm -rf '+name+'.ms.rawBPsource_*.png')
	plotms(vis=name+'.ms', 
	       field='J0522*',
	       xaxis='frequency', yaxis='amp',
	       selectdata=T, spw='17,19,21,23', 
	       avgtime='1e8',avgscan=T,
	       coloraxis='baseline',
	       antenna=refantenna+'&*',
	       ydatacolumn='data',
               showgui=F,
               plotfile=name+'.ms.rawBPsource_amp.png')
	plotms(vis=name+'.ms', 
	       field='J0522*',
	       xaxis='frequency', yaxis='phase',
	       selectdata=T, spw='17,19,21,23',
	       avgtime='1e8',avgscan=T,
	       coloraxis='baseline',
	       antenna=refantenna+'&*',
	       ydatacolumn='data',
               showgui=F,
               plotfile=name+'.ms.rawBPsource_phase.png')

####################################
# Apply WVR, Tsys and antenna position corrections 
mystep = 6
if(mystep in thesteps):
    casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
    print 'Step ', mystep, step_title[mystep]

    from recipes.almahelpers import tsysspwmap
    tsysmap = tsysspwmap(vis = basename[0]+'.ms', tsystable = basename[0]+'.ms.tsys')
    
    for field in ['J0522-364','J0522-3627','J0648-3044','Vy_CMa']:
        name='uid___A002_X6cdf83_X1118'
        applycal(vis='uid___A002_X6cdf83_X1118.ms',
                 spw='17,19,21,23',
                 field=field,
                 gainfield=[field,'',''],
                 interp=['linear,linear'],
                 gaintable=[name+'.ms.tsys',name+'.ms.wvr.smooth',name+'.ms.antpos'],
                 flagbackup=F, calwt=T, 
                 spwmap = [tsysmap,[],[]])

    for field in ['Pallas','J0522-3627','J0648-3044','Vy_CMa']:
        name='uid___A002_X6cdf83_X12ca'
        applycal(vis = 'uid___A002_X6cdf83_X12ca.ms',
                 spw='17,19,21,23',
                 field=field,
                 gainfield=[field,'',''],
                 interp=['linear,linear'],
                 gaintable=[name+'.ms.tsys',name+'.ms.wvr.smooth',name+'.ms.antpos'],
                 flagbackup=F, calwt=T, 
                 spwmap = [tsysmap,[],[]])

    for field in ['Pallas','J0522-3627','J0648-3044','Vy_CMa']:
        name='uid___A002_X6d0f96_X2051'
        applycal(vis = 'uid___A002_X6d0f96_X2051.ms',
                 spw='17,19,21,23',
                 field=field,
                 gainfield=[field,'',''],
                 interp=['linear,linear'],
                 gaintable=[name+'.ms.tsys',name+'.ms.wvr.smooth',name+'.ms.antpos'],
                 flagbackup=F, calwt=T, 
                 spwmap = [tsysmap,[],[]])

###############################################
# Split out science spw with time averaging
mystep = 7
if(mystep in thesteps):
    casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
    print 'Step ', mystep, step_title[mystep]

    for name in basename:
        os.system('rm -rf '+ name+'.ms.split') 
        split(vis = name+'.ms',
              outputvis = name+'.ms.split',
              datacolumn = 'corrected',
              spw = '17,19,21,23',
              timebin = '6.048s',
              keepflags = T)
  
#########################################
# Listobs, clear pointing table, and save original flags
mystep=8
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  for name in basename:
      os.system('rm -rf '+name+'.ms.split.listobs.txt')
      listobs(vis = name+'.ms.split',
              listfile = name+'.ms.split.listobs.txt')

# Remove POINTING tables
  for name in basename:
      tb.open(name+'.ms.split/POINTING', nomodify = False)
      a = tb.rownumbers()
      tb.removerows(a)
      tb.close()

# Store pre-flagging state
  for name in basename:
      flagmanager(vis = name+'.ms.split',
                  mode = 'save',
                  versionname = 'Pre-flagging')

#######
# Plot visibilities of bandpass calibration source after applying instrumental calibration
mystep = 9
if(mystep in thesteps):
    casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
    print 'Step ', mystep, step_title[mystep]

    for name in basename:
        os.system('rm -rf '+name+'.ms.split.corrBPsource_*.png')
	plotms(vis=name+'.ms.split', 
	       field='J0522*',
	       xaxis='frequency', yaxis='amp',
	       selectdata=T, spw='0,1,2,3', 
	       avgtime='1e8',avgscan=T,
	       coloraxis='baseline',
               antenna=refantenna+'&*',
               showgui=F,
               plotfile=name+'.ms.split.corrBPsource_amp.png')
	plotms(vis=name+'.ms.split', 
	       field='J0522*',
	       xaxis='frequency', yaxis='phase',
	       selectdata=T, spw='0,1,2,3',
	       avgtime='1e8',avgscan=T,
	       coloraxis='baseline',
               antenna=refantenna+'&*',
               showgui=F,
               plotfile=name+'.ms.split.corrBPsource_phase.png')

    print 'You should see that the scatter in the data is slightly less, but a few bad\badpoints appear. These are included in the flagging commands below, but you can\ninspect manually in plotms and modify flagging commands if wanted.'

#################
# Flagging shadowed data and other bad data and back up flags
mystep = 10
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  for name in basename:
      flagdata(vis = name+'.ms.split',
               mode = 'shadow',
               flagbackup = F)
  
  flagdata(vis = 'uid___A002_X6cdf83_X1118.ms.split',
           mode='manual',
           correlation='YY',
           antenna='DV05&*',
           spw='0:893~897;1917~1921;2941~2945')

  flagdata(vis='uid___A002_X6cdf83_X12ca.ms.split',
           mode='manual',
           correlation='YY',
           antenna='DV05&*',
           spw='0:893~897;1917~1921;2941~2945')

# End channels
  for name in basename:
      flagdata(vis = name+'.ms.split',
               mode='manual',
               spw='2:0,2:3839,3:0,3:3839',
               flagbackup = F)

# Bad data on DA41
  flagdata(vis='id___A002_X6d0f96_X2051.ms.split',
           mode='manual',
           antenna='DA41&*',
           timerange='12:51:00~12:52:00')

# DV24 scan 10 
  flagdata(vis = 'uid___A002_X6cdf83_X1118.ms.split',
           mode='manual',
           antenna='DV24&*',
           scan='10')

  for name in basename:
      flagmanager(vis = name+'.ms.split',
                  mode = 'save',
                  versionname = 'BeforeBandpassCalibration')

 
#################################
# Bandpass calibration
# First perform time-dependent calibration, averaging the central channels
mystep = 11
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  for name in basename:
      os.system('rm -rf cal-'+name+'-BPint.Gap')
      gaincal(vis=name+'.ms.split',
              caltable='cal-'+name+'-BPint.Gap',
              spw='*:1536~2304',field='J0522*',
              selectdata=F, solint='int',
              refant=refantenna, calmode='ap')

  if(calplots): # make some plots if calplots=True     
      for name in basename:
          os.system('rm -rf cal-'+name+'-phase-time-?-BPint-Gap.png')
          os.system('rm -rf cal-'+name+'-amp-time-?-BPint-Gap.png')
          plotcal(caltable='cal-'+name+'-BPint.Gap',
                  xaxis = 'time', yaxis = 'phase',poln='X',
                  plotsymbol='o', plotrange = [0,0,-180,180],
                  iteration = 'spw',
                  figfile='cal-'+name+'-phase-time-X-BPint-Gap.png',
                  subplot = 221)
          plotcal(caltable='cal-'+name+'-BPint.Gap',
                  xaxis = 'time', yaxis = 'phase',poln='Y',
                  plotsymbol='o', plotrange = [0,0,-180,180],
                  iteration = 'spw',
                  figfile='cal-'+name+'-phase-time-Y-BPint-Gap.png',
                  subplot = 221)
          plotcal(caltable='cal-'+name+'-BPint.Gap',
                  xaxis = 'time', yaxis = 'amp',poln='X',
                  plotsymbol='o',
                  iteration = 'spw',
                  figfile='cal-'+name+'-amp-time-X-BPint-Gap.png',
                  subplot = 221)
          plotcal(caltable='cal-'+name+'-BPint.Gap',
                  xaxis = 'time', yaxis = 'amp',poln='Y',
                  plotsymbol='o', 
                  iteration = 'spw',
                  figfile='cal-'+name+'-amp-time-Y-BPint-Gap.png',
                  subplot = 221)

  for name in basename:
      os.system('rm -rf cal-'+name+'.B1')
      bandpass(vis=name+'.ms.split',
               caltable='cal-'+name+'.B1',
               field='J0522*', 
               bandtype='B', 
               fillgaps=10, solnorm = T, 
               solint='inf,20ch', 
               refant=refantenna,
               gaintable='cal-'+name+'-BPint.Gap')


  if(calplots): # make some plots if calplots=True
      for name in basename:
          os.system('rm -rf cal-'+name+'-phase_spw*-B1.png')
          os.system('rm -rf cal-'+name+'-amp_spw*-B1.png')
          for spw in range(2):
              for ant in range(3):
                  plotcal(caltable = 'cal-'+name+'.B1',
                      xaxis='freq', yaxis='phase', spw=str(spw*2)+'~'+str(spw*2+1), 
                      antenna=str(ant*8)+'~'+str(ant*8+7),
                      iteration='antenna',subplot=421, 
                      fontsize=7.0,
                      plotrange = [0,0,-180,180],
                      figfile='cal-'+name+'-phase_spw'+str(spw*2)+'-'+str(spw*2+1)+'_ant'+str(ant*8)+'-'+str(ant*8+7)+'-B1.png')
                  plotcal(caltable = 'cal-'+name+'.B1',
                      xaxis='freq', yaxis='amp', spw=str(spw*2)+'~'+str(spw*2+1), 
                      antenna=str(ant*8)+'~'+str(ant*8+7),
                      iteration='antenna',subplot=421, 
                      fontsize=7.0,
                      figfile='cal-'+name+'-amp_spw'+str(spw*2)+'-'+str(spw*2+1)+'_ant'+str(ant*8)+'-'+str(ant*8+7)+'-B1.png')
#################
# Putting in model for flux calibrator Pallas
# This is only present in two of the data sets, so we use it to derive the flux density
# of the bandpass calibrator and use this to set the scale for the other data set
# Use the plots to decide the uvrange to use in calibration of Pallas
mystep = 12
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  setjy(vis = 'uid___A002_X6cdf83_X12ca.ms.split',
    field = 'Pallas',
    spw = '0,1,2,3',
    standard = 'Butler-JPL-Horizons 2012')
  os.system('rm -rf uid___A002_X6cdf83_X12ca.ms.split.setjy.Pallas.png') 
  plotms(vis = 'uid___A002_X6cdf83_X12ca.ms.split',
         xaxis = 'uvdist',
         yaxis = 'amp',
         ydatacolumn = 'model',
         field = 'Pallas',
         spw = '0,1,2,3',
         avgchannel = '9999',
         coloraxis = 'spw',
         plotfile = 'uid___A002_X6cdf83_X12ca.ms.split.setjy.Pallas.png')
 
  setjy(vis = 'uid___A002_X6d0f96_X2051.ms.split',
    field = '2', # Pallas
    spw = '0,1,2,3',
    standard = 'Butler-JPL-Horizons 2012')
  os.system('rm -rf uid___A002_X6d0f96_X2051.ms.split.setjy.Pallas.png') 
  plotms(vis = 'uid___A002_X6d0f96_X2051.ms.split',
         xaxis = 'uvdist',
         yaxis = 'amp',
         ydatacolumn = 'model',
         field = 'Pallas',
         spw = '0,1,2,3',
         avgchannel = '9999',
         coloraxis = 'spw',
         plotfile = 'uid___A002_X6d0f96_X2051.ms.split.setjy.Pallas.png')

#################################
# Setting the flux scale
# 
mystep = 13
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

# Remove any previous models for QSOs
  for name in basename:
      delmod(vis=name+'.ms.split', field='J*')

# Use Pallas where present
  os.system('rm -rf cal-uid___A002_X6cdf83_X12ca.Gp_short') 
  gaincal(vis = 'uid___A002_X6cdf83_X12ca.ms.split',
    caltable = 'cal-uid___A002_X6cdf83_X12ca.Gp_short',
    field = 'Pallas',
    selectdata = T,
    uvrange = '0~700',
    solint = 'int',
    refant = refantenna,
    gaintype = 'G',
    calmode = 'p',
    gaintable = 'cal-uid___A002_X6cdf83_X12ca.B1')
  
  gaincal(vis = 'uid___A002_X6cdf83_X12ca.ms.split',
    caltable = 'cal-uid___A002_X6cdf83_X12ca.Gp_short',
    field = 'J*',
    selectdata = T,
    solint = 'int',
    refant = refantenna,
    gaintype = 'G',
    calmode = 'p',
    append = T,
    gaintable = 'cal-uid___A002_X6cdf83_X12ca.B1')

  os.system('rm -rf cal-uid___A002_X6cdf83_X12ca.Gap_short') 
  gaincal(vis = 'uid___A002_X6cdf83_X12ca.ms.split',
    caltable = 'cal-uid___A002_X6cdf83_X12ca.Gap_short',
    field = 'Pallas,J*',
    selectdata = T,
    solint = 'inf',
    refant = refantenna,
    uvrange = '0~700',
    gaintype = 'T',
    calmode = 'ap',
    gaintable = ['cal-uid___A002_X6cdf83_X12ca.B1', 'cal-uid___A002_X6cdf83_X12ca.Gp_short'])

  os.system('rm -rf cal-uid___A002_X6cdf83_X12ca.Gap_short_flux') 
  F1=fluxscale(vis = 'uid___A002_X6cdf83_X12ca.ms.split',
    caltable = 'cal-uid___A002_X6cdf83_X12ca.Gap_short',
    fluxtable = 'cal-uid___A002_X6cdf83_X12ca.Gap_short_flux',
    reference = 'Pallas') 

#
  os.system('rm -rf cal-uid___A002_X6d0f96_X2051.Gp_short') 
  gaincal(vis = 'uid___A002_X6d0f96_X2051.ms.split',
    caltable = 'cal-uid___A002_X6d0f96_X2051.Gp_short',
    field = 'Pallas', 
    selectdata = T,
    uvrange = '0~700',
    solint = 'int',
    refant = refantenna,
    gaintype = 'G',
    calmode = 'p',
    gaintable = 'cal-uid___A002_X6d0f96_X2051.B1')
  
  gaincal(vis = 'uid___A002_X6d0f96_X2051.ms.split',
    caltable = 'cal-uid___A002_X6d0f96_X2051.Gp_short',
    field = 'J*', 
    selectdata = T,
    solint = 'int',
    refant = refantenna,
    gaintype = 'G',
    calmode = 'p',
    append = T,
    gaintable = 'cal-uid___A002_X6d0f96_X2051.B1')

  os.system('rm -rf cal-uid___A002_X6d0f96_X2051.Gap_short') 
  gaincal(vis = 'uid___A002_X6d0f96_X2051.ms.split',
    caltable = 'cal-uid___A002_X6d0f96_X2051.Gap_short',
    field = 'Pallas,J*', 
    selectdata = T,
    solint = 'inf',
    refant = refantenna,
    uvrange = '0~700',
    gaintype = 'T',
    calmode = 'ap',
    gaintable = ['cal-uid___A002_X6d0f96_X2051.B1', 'cal-uid___A002_X6d0f96_X2051.Gp_short'])

  os.system('rm -rf cal-uid___A002_X6d0f96_X2051.Gap_short_flux') 
  F2=fluxscale(vis = 'uid___A002_X6d0f96_X2051.ms.split',
    caltable = 'cal-uid___A002_X6d0f96_X2051.Gap_short',
    fluxtable = 'cal-uid___A002_X6d0f96_X2051.Gap_short_flux',
    reference = 'Pallas')

  J0522_flux=(F1['0']['fitFluxd']/(F1['0']['fitFluxdErr'])**2 + F2['0']['fitFluxd']/(F2['0']['fitFluxdErr'])**2)/((F1['0']['fitFluxdErr'])**-2 + (F2['0']['fitFluxdErr'])**-2)

  # if (version < '4.2.1'):
  # Due to bug, this value has to be calculated from logger messages in CASA 4.2.0
   #    J0522_flux=4.750479

  J0522_rfreq=1e-9*(F1['0']['fitRefFreq']+F2['0']['fitRefFreq'])/2.
  J0522_spidx=(F1['0']['spidx'][1]/(F1['0']['spidxerr'][1])**2 + F2['0']['spidx'][1]/(F2['0']['spidxerr'][1])**2)/((F1['0']['spidxerr'][1])**-2 + (F2['0']['spidxerr'][1])**-2)

  J0648_flux=(F1['3']['fitFluxd']/(F1['3']['fitFluxdErr'])**2 + F2['3']['fitFluxd']/(F2['3']['fitFluxdErr'])**2)/((F1['3']['fitFluxdErr'])**-2 + (F2['3']['fitFluxdErr'])**-2)
  J0648_rfreq=1e-9*(F1['3']['fitRefFreq']+F2['3']['fitRefFreq'])/2.
  J0648_spidx=(F1['3']['spidx'][1]/(F1['3']['spidxerr'][1])**2 + F2['3']['spidx'][1]/(F2['3']['spidxerr'][1])**2)/((F1['3']['spidxerr'][1])**-2 + (F2['3']['spidxerr'][1])**-2)

  print 'Flux density of J0522-3627 is %5.3f at %7.3f GHz, spectral index %4.2f' % (J0522_flux, J0522_rfreq, J0522_spidx)
  print 'Flux density of J0648-3044 is %5.3f at %7.3f GHz, spectral index %4.2f' % (J0648_flux, J0648_rfreq, J0648_spidx)
  print 'See logger for individual results and uncertainties.'

  for name in basename:
      setjy(vis=name+'.ms.split',
            field='J0522*',
            standard='manual',
            fluxdensity=[J0522_flux,0,0,0],
            spix=J0522_spidx,
            reffreq=str(J0522_rfreq)+'GHz')      
      setjy(vis=name+'.ms.split',
            field='J0648-3044',
            standard='manual',
            fluxdensity=[J0648_flux,0,0,0],
            spix=J0648_spidx,
            reffreq=str(J0648_rfreq)+'GHz')

# These are the values I got
# Flux density of J0522-3627 is 4.748 at 316.093 GHz, spectral index -0.49
# Flux density of J0648-3044 is 0.430 at 316.093 GHz, spectral index -0.83

#################
# Time-dependent calibration 
mystep = 14
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

# phase calibration per integration to apply to cal sources
  for name in basename:
      os.system('rm -rf cal-'+name+'.Gp_int')
      gaincal(vis=name+'.ms.split',
              caltable = 'cal-'+name+'.Gp_int',
              field='J*',
              solint='int',
              refant=refantenna,
              calmode='p',
              gaintable = 'cal-'+name+'.B1')

  if(calplots): # make some plots if calplots=True
      for name in basename:
          os.system('rm -rf cal-'+name+'-phase-time-?-Gp_int.png')
          plotcal(caltable='cal-'+name+'.Gp_int',
                  xaxis = 'time', yaxis = 'phase',poln='X',
                  plotsymbol='o', plotrange = [0,0,-180,180],
                  iteration = 'spw',
                  figfile='cal-'+name+'-phase-time-X-Gp_int.png',
                  subplot = 221)
          plotcal(caltable='cal-'+name+'.Gp_int',
                  xaxis = 'time', yaxis = 'phase',poln='Y',
                  plotsymbol='o', plotrange = [0,0,-180,180],
                  iteration = 'spw',
                  figfile='cal-'+name+'-phase-time-Y-Gp_int.png',
                  subplot = 221)
 
# phase calibration per scan for phase-ref to apply to target
  for name in basename:
      os.system('rm -rf cal-'+name+'.Gp_inf')
      gaincal(vis=name+'.ms.split',
              caltable = 'cal-'+name+'.Gp_inf',
              field='J0648-3044',
              solint='inf',
              refant=refantenna,
              calmode='p',
              gaintable = 'cal-'+name+'.B1')

# amp cal per scan
  for name in basename:
      os.system('rm -rf cal-'+name+'.Gap_inf')
      gaincal(vis=name+'.ms.split',
              caltable = 'cal-'+name+'.Gap_inf',
              field='J*',
              solint='inf',
              refant=refantenna,
              gaintype='T',
              calmode='ap',
              gaintable = ['cal-'+name+'.B1','cal-'+name+'.Gp_int'])

  if(calplots): # make some plots if calplots=True
      for name in basename:
          os.system('rm -rf cal-'+name+'-amp-time-Gap_inf.png')
          plotcal(caltable='cal-'+name+'.Gap_inf',
                  xaxis = 'time', yaxis = 'amp',
                  plotsymbol='o',
                  iteration = 'spw',
                  figfile='cal-'+name+'-amp-time-Gap_inf.png',
                  subplot = 221)
 
#################
# Apply calibration
mystep = 15
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  for name in basename:
      for f in ['J0522*','J0648-3044']:
          applycal(vis=name+'.ms.split',
                   field=f,
                   gaintable=['cal-'+name+'.B1','cal-'+name+'.Gp_int','cal-'+name+'.Gap_inf'],
                   gainfield=['',f,f],
                   interp='linear,linear',
                   calwt=F,flagbackup=F)

  for name in basename:
      applycal(vis=name+'.ms.split',
               field='V*',
               gaintable=['cal-'+name+'.B1','cal-'+name+'.Gp_inf','cal-'+name+'.Gap_inf'],
               gainfield=['','J0648-3044','J0648-3044'],
               interp='linear,linear',
               calwt=F,flagbackup=F)

#######
# Plot phase-ref visibilities after applying instrumental calibration
mystep = 16
if(mystep in thesteps):
    casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
    print 'Step ', mystep, step_title[mystep]

    for name in basename:
        os.system('rm -rf '+name+'.ms.split.J0648-3044_*.png')
	plotms(vis=name+'.ms.split', 
	       field='J0648-3044',
	       xaxis='time', yaxis='amp',
	       selectdata=T, spw='0,1,2,3', 
	       avgtime='30s',avgchannel='3840',
	       coloraxis='spw',
               ydatacolumn='corrected',
               antenna=refantenna+'&*',
               showgui=F,
               plotfile=name+'.ms.split.J0648-3044_amp.png')
	plotms(vis=name+'.ms.split', 
	       field='J0648-3044',
	       xaxis='time', yaxis='phase',
	       selectdata=T, spw='0,1,2,3',
	       avgtime='30s',avgchannel='3840',
	       coloraxis='spw',
               ydatacolumn='corrected',
               antenna=refantenna+'&*',
               showgui=F,
               plotfile=name+'.ms.split.J0648-3044_phase.png')


###########################
# Split out the science data, concatenate and transform to lsrk to allow self-calibration
mystep = 17
if(mystep in thesteps):
    casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
    print 'Step ', mystep, step_title[mystep]

    vislist=[]
    for name in basename:
            os.system('rm -rf '+name+'_VYCMa_321.ms')
            mstransform(vis=name+'.ms.split',
            outputvis=name+'_VYCMa_321.ms',datacolumn='corrected',
            field='V*') 
            vislist.append(name+'_VYCMa_321.ms')

    os.system('rm -rf VYCMa_321_origspw.ms')
    concat(vis=vislist,
           concatvis='VYCMa_321_origspw.ms')
       
# Combine and regrid each group of spw with similar frequencies 
    vislist=[]
    sps=['0,4,8','1,5,9','2,6,10','3,7,11']
    for s in range(4):
        os.system('rm -rf VYCMa_321_spw'+str(s)+'.ms')
        mstransform(vis='VYCMa_321_origspw.ms',
            outputvis='VYCMa_321_spw'+str(s)+'.ms',datacolumn='data',
            spw=sps[s],
            field='V*',combinespws=T,regridms=T,
            mode='frequency',
            outframe='lsrk')
        vislist.append('VYCMa_321_spw'+str(s)+'.ms')

# Combine all regridded data
    os.system('rm -rf VYCMa_Band7_321.ms')
    concat(vis=vislist,
           concatvis='VYCMa_Band7_321.ms')
    
# Do listobs and write info to a text file
    os.system('rm VYCMa_Band7_321_listobs.txt')
    listobs(vis='VYCMa_Band7_321.ms', listfile='VYCMa_Band7_321_listobs.txt', verbose=True)
    print '<< Printed listobs output to VYCMa_Band7_321_listobs.txt'

#----------------------------------------------------------------------------------
#----- End of calibration script.
#----- To continue, see imaging script "VYCMa_Band7_321GHz_Imaging.py"
#----------------------------------------------------------------------------------
